# A queue manager, which happens to use the file system

from pathlib import Path
from tempfile import TemporaryDirectory
import os

from .file import File
from .queue import Queue


class Root:

    def __init__(self, path=None):
        if path:
            self.path = path
        self._files = {}
        self._queues = {}

    @property
    def path(self):
        if not hasattr(self, '_path'):
            env_var = os.environ.get('BUSY_ROOT')
            self._path = Path(env_var if env_var else Path.home() / '.busy')
            if not self._path.is_dir():
                self._path.mkdir()
        return self._path

    @path.setter
    def path(self, value):
        assert not hasattr(self, '_path')
        path = Path(value) if isinstance(value, str) else value
        assert isinstance(path, Path) and path.is_dir()
        self._path = path

    def get_queue(self, key=None):
        key = key or Queue.default_key
        if key not in self._queues:
            queueclass = Queue.subclass(key)
            queuefile = File(self.path / f'{key}.txt')
            self._files[key] = queuefile
            self._queues[key] = queueclass(self)
            self._queues[key].add(*queuefile.read(queueclass.itemclass))
        return self._queues[key]

    def save(self):
        while self._queues:
            key, queue = self._queues.popitem()
            items = queue.all()
            self._files[key].save(*items)



from unittest import TestCase
from tempfile import TemporaryDirectory
from pathlib import Path
from unittest import mock
from io import StringIO
from datetime import date as Date

from busy.plugins.todo import Task
from busy.commander import Commander

class TestCommandDrop(TestCase):

    def test_drop(self):
        with TemporaryDirectory() as t:
            p = Path(t, 'tasks.txt')
            p.write_text('a\nb\nc\nd')
            c = Commander(root=t)
            c.handle('drop','2','4')
            o = p.read_text()
            self.assertEqual(o, 'a\nc\nb\nd\n')

    def test_pop(self):
        with TemporaryDirectory() as t:
            p = Path(t, 'tasks.txt')
            p.write_text('a\nb\nc\nd')
            c = Commander(root=t)
            c.handle('pop','2','4')
            o = p.read_text()
            self.assertEqual(o, 'b\nd\na\nc\n')

    def test_no_output(self):
        with TemporaryDirectory() as t:
            p = Path(t, 'tasks.txt')
            p.write_text('a\nb\n')
            c = Commander(root=t)
            o = c.handle('drop','1')
            self.assertEqual(o, None)



import sys

from .commander import Commander


def main():
    try:
        output = Commander().handle(*sys.argv[1:])
    except (RuntimeError, ValueError) as error:
        output = f"Error: {str(error)}"
    if output:
        print(output)


if __name__ == '__main__':
    main()
